@@ -105,6 +105,8 @@ struct NautilusDirectoryDetails
 	LinkInfoReadState *link_info_read_state;
 
 	GList *file_operations_in_progress; /* list of FileOperation * */
+
+	GHashTable *hidden_file_hash;
 };
 
 /* A request for information about one or more files. */
@@ -125,14 +127,14 @@ NautilusDirectory *nautilus_directory_get_existing                    (const cha
 void               nautilus_directory_async_state_changed             (NautilusDirectory         *directory);
 void               nautilus_directory_call_when_ready_internal        (NautilusDirectory         *directory,
 								       NautilusFile              *file,
-								       GList                     *file_attributes,
+								       NautilusFileAttributes     file_attributes,
 								       gboolean                   wait_for_file_list,
 								       NautilusDirectoryCallback  directory_callback,
 								       NautilusFileCallback       file_callback,
 								       gpointer                   callback_data);
 gboolean           nautilus_directory_check_if_ready_internal         (NautilusDirectory         *directory,
 								       NautilusFile              *file,
-								       GList                     *file_attributes);
+								       NautilusFileAttributes     file_attributes);
 void               nautilus_directory_cancel_callback_internal        (NautilusDirectory         *directory,
 								       NautilusFile              *file,
 								       NautilusDirectoryCallback  directory_callback,
@@ -143,7 +145,7 @@ void               nautilus_directory_monitor_add_internal            (NautilusD
 								       gconstpointer              client,
 								       gboolean                   monitor_hidden_files,
 								       gboolean                   monitor_backup_files,
-								       GList                     *attributes,
+								       NautilusFileAttributes     attributes,
 								       NautilusDirectoryCallback  callback,
 								       gpointer                   callback_data);
 void               nautilus_directory_monitor_remove_internal         (NautilusDirectory         *directory,
@@ -162,10 +164,10 @@ void               nautilus_directory_stop_monitoring_file_list       (NautilusD
 void               nautilus_directory_cancel                          (NautilusDirectory         *directory);
 void               nautilus_async_destroying_file                     (NautilusFile              *file);
 void               nautilus_directory_force_reload_internal           (NautilusDirectory         *directory,
-								       GList                     *file_attributes);
+								       NautilusFileAttributes     file_attributes);
 void               nautilus_directory_cancel_loading_file_attributes  (NautilusDirectory         *directory,
 								       NautilusFile              *file,
-								       GList                     *file_attributes);
+								       NautilusFileAttributes     file_attributes);
 
 /* Calls shared between directory, file, and async. code. */
 void               nautilus_directory_emit_files_added                (NautilusDirectory         *directory,
@@ -182,7 +184,7 @@ NautilusDirectory *nautilus_directory_get_internal                    (const cha
 								       gboolean                   create);
 char *             nautilus_directory_get_name_for_self_as_new_file   (NautilusDirectory         *directory);
 void               nautilus_directory_set_up_request                  (Request                   *request,
-								       GList                     *file_attributes);
+								       NautilusFileAttributes     file_attributes);
 
 /* Interface to the file list. */
 NautilusFile *     nautilus_directory_find_file_by_name               (NautilusDirectory         *directory,
